function [opt_input opt_output opt_fau] = generate_opts(sig_type,fau_type)
% GENERATE_OPTS option for input, output and fault generation
%   It generates the option used by input_generation, signal_generation and
%   fault generation to generate a faulty couple of singnal. fau_type input
%   is optional and should be defined only if we want to introduce a fault
%   in the data
%
%   Input:
%       sig_type.n_input: number of input to be considered
%       sig_type.gene: type of the signal generated
%           'pendulum'      = pendulum discretization (fixed lenght and mass)
%           'vanderpol'     = van der pol oscillator discretization
%           'linear'        = linear generic ARX model
%               requires sig_type.na order of the autoregressive part
%                        sig_type.nb order of the exogenous part
%                        sig_type.nc flag for constant
%               optional sig_type.nonlin if present introduce nonlinearity
%                   in the linear generation
%       sig_type.batch: lenght of each batch to be considered in the
%       following
%       sig_type.n_train_batch: number of training batches to be simulated
%       sig_type.n_test_batch: number of test batches to be simulated
%       sig_type.hor: lenght of the signal to be generated
%       sig_type.noise_std: noise standard deviation of the output
%       fau_type.stru: vector specifying the structured nature (with
%           dynamic or not) of each fault to inject
%       fau_type.gene: vector fo the type of the faults generated
%           (see sig_type.gene)
%       fau_type.int: vector of faults intensity rate
%   Output:
%       opt_input.hor: number of samples to be generated in the input
%           generation process
%       opt_output.noise_std: noise standard deviation
%       opt_output.signal_type: type of the generated signal
%       if sig_type.gene = 'pendulum'
%           opt_output.l: pendulum length
%           opt_output.m: pendulum mass
%       if sig_type.gene = 'linear'
%           opt_output.na: order of the autoregressive part
%           opt_output.nb: order of the exogenous part
%           opt_output.acoeff = coefficients of the autoregressive part
%           opt_output.bcoeff: coefficients of the exogenous part       
%           opt_output.ccoeff: constant of the process
%           opt_output.nonlin: non linearity to introduce
%       opt_output.hor: number of samples to be generated
%       opt_output.n_train: number of samples in the training set
%       opt_fau.stru = fau_type.stru; (TODO)
%       opt_fau.fault_type = fau_type.gene;
%       opt_fau.not_fault = 5 + eff_batch * sig_type.n_train_batch;
%       opt_fau.final_intensity_rate = fau_type.int;

opt_input = sig_type.opt_input;


switch sig_type.gene
    case 'pendulum'
        opt_output.l = 10; % pendulum lenght
        opt_output.m = 1; % mass at the end of the pendulum
        eff_batch = 2 + sig_type.batch;
    case 'vanderpol'
        eff_batch = 2 + sig_type.batch;
    otherwise
        opt_output.na = sig_type.na;
        opt_output.nb = sig_type.nb;
        opt_output.acoeff = rand(sig_type.na,1)-0.5;
        opt_output.bcoeff = rand(sig_type.nb,sig_type.n_input)-0.5;
        if sig_type.nc == 1
            opt_output.ccoeff = rand()-0.5;
        else
            opt_output.ccoeff = 0;
        end
        
        if isfield(sig_type,'nab'); %for quadratic (beta)
            opt_output.nab = sig_type.nab;
            opt_output.abcoeff = rand(sig_type.nab,1)-0.5;
            max_ord = max([sig_type.na sig_type.nb sig_type.nab]);
        else
            max_ord = max([sig_type.na sig_type.nb]);            
        end
        eff_batch = max_ord + sig_type.batch;
end
opt_output.noise_std = sig_type.noise_std;
opt_output.signal_type = sig_type.gene;
if isfield(sig_type,'nonlin')
    opt_output.nonlin = sig_type.nonlin;
end

% generates 5 additional sample, discarded in what follows because of possible instability
opt_input.hor = 5 + eff_batch * (sig_type.n_train_batch + sig_type.n_test_batch);
opt_output.hor = opt_input.hor;
opt_output.n_train = 5 + eff_batch * sig_type.n_train_batch;

if isempty(fau_type)
    % No fault
    opt_fau = [];
else
    % Fault
    assert(mod(sig_type.n_test_batch,length(fau_type.gene)) == 0);
    opt_fau.stru = fau_type.stru;
    opt_fau.fault_type = fau_type.gene;
    opt_fau.not_fault = 5 + eff_batch * sig_type.n_train_batch;
    opt_fau.final_intensity_rate = fau_type.int;
end
