function [y opt_output] = signal_generation(x,opt_output,varargin)
% SIGNAL_GENERATION signal generation
%   Generates a vector of values representing the output of a process 
%   (monodimensional)
%
%   Input:
%       x: input of the dynamic system to simulate
%       opt_output.noise_std: noise standard deviation
%       opt_output.signal_type: type of the generated signal
%       if sig_type.gene = 'pendulum'
%           opt_output.l: pendulum length
%           opt_output.m: pendulum mass
%       if sig_type.gene = 'linear'
%           opt_output.na: order of the autoregressive part
%           opt_output.nb: order of the exogenous part
%           opt_output.acoeff = coefficients of the autoregressive part
%           opt_output.bcoeff: coefficients of the exogenous part       
%           opt_output.nonlin: non linearity to introduce
%       opt_output.hor: number of samples to be generated
%       opt_output.n_train: number of samples in the training set
%       varargin{1}: initial output points (optional)
%
%   Output:
%       y: vector containing the values of the exogenous part of a process
%       opt_output: enhanced option of the output useful in the fault
%       injection process with inject_fault

% Fault free output initialization
y = zeros(1,opt_output.hor);
if nargin > 2
    init_point = varargin{1};
    y(1:opt_output.na) = init_point;
end

% Signal generation
switch opt_output.signal_type
    case 'linear' % Linear data generation
        opt_output.max_order = max([opt_output.na opt_output.nb]);
        for i = (opt_output.max_order+1):opt_output.hor
            input_contr = opt_output.bcoeff .* x(:,i-opt_output.nb:i-1)';
            y(i) = opt_output.acoeff' * y(i-opt_output.na:i-1)' + sum(input_contr(:));
            if isfield(opt_output,'nonlin')
                y(i) = feval(opt_output.nonlin,y(i));
            end
        end
    case 'pendulum' % pendulum
        opt_output.na = 2;
        opt_output.nb = 2;
        opt_output.ccoeff = 0;
        opt_output.max_order = max([opt_output.na opt_output.nb]);
        g = 9.8;
        for i = 3:opt_output.hor
%            y(i) = mod(- g / opt_sig.l * sin(y(i-2)) + 2 * y(i-2) - y(i-2) + x(i-2) / (y(i-2)+0.01) / opt_sig.l / opt_sig.m, 2 * pi);
            y(i) = mod(- g / opt_output.l * sin(y(i-2)) + 2 * y(i-2) - y(i-2) + x(i-2) / opt_output.l / opt_output.m, 2 * pi);
        end
    case 'vanderpol' % Van der pol oscillator
        opt_output.na = 2;
        opt_output.nb = 0;
        opt_output.max_order = max([opt_output.na opt_output.nb]);
        y1 = zeros(1,opt_output.hor);
        y2 = zeros(1,opt_output.hor);
        y1(1) = randn(1);        
        y2(1) = randn(1);
        deltat = 0.01;
        for i = 2:opt_output.hor
            y1(i) = y1(i-1) * deltat + y2(i-1);
            y2(i) = y2(i-1) + y2(i-1) * (1 - y1(i-1)^2) * deltat - y1(i-1) * deltat + randn(1)*0.1;
        end
        y = y2;
    case 'quadratic' % Quadratic model (beta)
        opt_output.max_order = max([opt_output.na opt_output.nb]);
        for i = (opt_output.max_order+1):opt_output.hor
            y(i) = opt_output.acoeff * y(i-opt_output.na:i-1)' + opt_output.bcoeff * x(i-opt_output.nb:i-1)' + ...
                opt_output.abcoeff * y(i-1) * x(i-1);
        end
    otherwise
        fprintf('%s\n','The selected type of signal generation is not supported.');
end

range = max(y(1+opt_output.na:end)) - min(y(1+opt_output.na:end));
y = y + randn(1,opt_output.hor) * opt_output.noise_std * range;

if (any(y == Inf) || any(isnan(y)))
    disp('Warning: the data generating process is not stable.');
end
