%  This is a demo file illustrating the usage of evolving clustering
%  algorithm for fault diagnosis
%
%
% Francesco Trovo'
% Politecnico di Milano
% January 2015
% francesco1.trovo@@polimi.it
%
% References
%
% [1] Alippi, C.; Roveri, M.; Trovo', F., "A Self-Building and 
% Cluster-Based Cognitive Fault Diagnosis System for Sensor Networks",
% Neural Networks and Learning Systems, IEEE Transactions on , vol.25, 
% no.6, pp.1021,1032, June 2014 DOI: 10.1109/TNNLS.2014.2303651
%
% [2] Alippi, C,; Roveri, M; Trovo', F., "A "Learning from Models" 
% Cognitive Fault Diagnosis System", Artificial Neural Networks and 
% Machine Learning -- ICANN 2012, LNCS 7553}, pp.305,312, October 2012, 
% Springer-Verlag DOI: 10.1007/978-3-642-33266-1_38
clc;
close all;
clear;

addpath(genpath('.'));

%% Data generation

sig_type.opt_input.type = 'random_walk';
sig_type.opt_input.noise_std = 0.1;
sig_type.opt_input.a = 0.4;

sig_type.n_input = 1;
sig_type.gene = 'linear';
%sig_type.nonlin = @(x)sin(x);
sig_type.coeff = 'random';
sig_type.na = 2;
sig_type.nb = 2;
sig_type.nc = 0;
sig_type.nab = 1;

sig_type.n_train_batch = 100;
sig_type.n_test_batch = 210;
sig_type.batch = 400;
sig_type.noise_std = 0.001;

fau_type.stru = [1 0 0];
fau_type.gene = {'none' 'abr_mul' 'abr_add'};
fau_type.int = [0 0.7 0.4];

[opt_input opt_output opt_fau] = generate_opts(sig_type,fau_type);
x = input_generation(opt_input);
[y opt_output] = signal_generation(x,opt_output);
[yf opt_fau] = fault_generation(x,y,opt_output,opt_fau);


opt_est.n_input = 1;
opt_est.batch = sig_type.batch;
opt_est.aord = 2;
opt_est.bord = 2;
opt_est.kord = 0;
opt_est.max_ord = max([opt_est.aord opt_est.bord + opt_est.kord]);
opt_est.eff_batch = opt_est.batch + opt_est.max_ord;
opt_est.const_input = 0;
opt_est.mode = 'arx';
opt_est.res_anal = 0;

%% Initialize cluster
clu = init_clust();

%Division into training and testing
opt_evo.n_train_batch = sig_type.n_train_batch;
opt_evo.n_test_batch = sig_type.n_test_batch;

% Update nominal state
opt_evo.up_nom = 1;


% Covariance update
opt_evo.up_cov = 'g';

% Confidence for spatial, temporal, etc...
opt_evo.alpha_s = 0.03;
opt_evo.alpha_m = 0.05;
opt_evo.eta_t = 1;

% Parameter for cluster creation
opt_evo.alpha_c = 0.10;
opt_evo.w = 0.5;
opt_evo.n_montec_sample = 1000;

% Parameters for r estimation
opt_evo.rad_calc = 'corr';
opt_evo.alpha_rho = 0.90;
opt_evo.alpha_beta = 0.90;
opt_evo.rho_zero = 0.95;

opt_evo.max_iter = 30;
opt_evo.eta_i = 1e-6;

% Additional options
opt_evo.verbose = 1;

%% Training phase
n_batch = 0;
for i = 1:opt_evo.n_train_batch
    n_batch = n_batch + 1;
    init_batch = 1+(n_batch-1) * opt_est.eff_batch;
    end_batch = n_batch * opt_est.eff_batch;
    
    x_batch = x(init_batch:end_batch);
    y_batch = yf(init_batch:end_batch);

    model = model_creation(x_batch,y_batch,opt_est);
    clu = clust_train(clu,model,n_batch,opt_evo);
    
end

opt_draw.batch_color = [0.7 0.7 0.7];
opt_draw.max_batch = 15;
opt_draw.labels = 0;
opt_draw.wait = 0.1;

ax = axes();
plot_clust_evo_3D(clu,ax,opt_draw);
grid on;
view(3);

%% Testing phase
for i = 1:opt_evo.n_test_batch
    n_batch = n_batch + 1;
    init_batch = 1+(n_batch-1) * opt_est.eff_batch;
    end_batch = n_batch * opt_est.eff_batch;
    
    x_batch = x(init_batch:end_batch);
    y_batch = yf(init_batch:end_batch);

    model = model_creation(x_batch,y_batch,opt_est);
    clu = clust_evol(clu,model,n_batch,opt_evo);
    
    plot_clust_evo_3D(clu,ax,opt_draw);
    pause(0.1)
end
