function model = model_creation(x,y,opt_est,varargin)
%MODEL_CREATION Estimates parameter vector on a input-output stream.
%   MODEL = MODEL_CREATION(X,Y,OPT_EST) estimates the parameter vector on 
%   the relationship input-outpur X-Y, options recorded in OPT_EST are used
%
%   Input:
%       x: input data (matrix N*n)
%       y: output data (matrix N*1)
%       opt_est.n_input: number of input (n)
%       opt_est.batch: length of the batch for coefficients estimation (N)
%       opt_est.aord: autoregressive order for the arx
%       opt_est.bord: order for the exogenous part of the arx
%       opt_est.kord: time delay for the input
%       opt_est.max_ord: maximum order of the cosidered coeffiecients
%       opt_est.const_input: flag for an artifical constrant input
%       opt_est.opt_est.mode: coefficients estionation mode
%           'arx' least square minimization for arx model
%           'rn' least square minimization for a reservoir network
%        opt_est.res_anal: residual analysis for the created model
%
%	Output:
%       model.res_fau: coefficients fo the model with the fault
%       model.eps_fau: residual in faulty condition
%       model.Z: input-output sequences
%       model.Y: output sequence

%   Copyright 2013-2015 Francesco Trovo' Politecnico di Milano
%   $Revision: 1.1$  $Date: 2015/01/14$

if ~isfield(opt_est,'kord')
    opt_est.kord = 1;
end

assert( length(y) / opt_est.eff_batch == 1);
y_cons = zeros(opt_est.aord, opt_est.batch);
x_cons = zeros(opt_est.bord * opt_est.n_input, opt_est.batch);

assert(opt_est.max_ord == max([opt_est.aord opt_est.bord+opt_est.kord]));
init_index = opt_est.max_ord + 1;
final_index = opt_est.eff_batch;
y_to_est = y( init_index : final_index );
for i = 1:opt_est.aord
    y_cons(i,:) = y( (init_index-i) : (final_index-i) );
end
for i = 1:(opt_est.bord)
    x_cons((1+(i-1)*opt_est.n_input):(i*opt_est.n_input),:) = x (:, (init_index-i+1-opt_est.kord) : (final_index-i+1-opt_est.kord));
end
if opt_est.const_input == 1
    const_input = ones(1,opt_est.batch);
    x_cons = [x_cons; const_input];
end

phitra = [ y_cons; x_cons ];
model.Z{1} = phitra;
model.Y{1} = y_to_est;


if strcmp(opt_est.mode, 'arx')
    model.res_fau = (phitra * phitra') \ (phitra * y_to_est');
    % model.res_faures_fau = pinv(phitra * phitra') * (phitra * y_to_est');
elseif strcmp(opt_est.mode, 'rn')
    network = varargin{1};
    network = rn_train(network,phitra,y_to_est',opt_est);
    model.res_fau = network.W_out';
end

n_coeff = opt_est.aord + opt_est.bord * opt_est.n_input + opt_est.const_input;

model.residuals = model.res_fau(1:n_coeff)' * model.Z{1} - model.Y{1};
model.eps_fau = sqrt(sum((model.residuals).^2)) / opt_est.batch;

model.param_cov = phitra * phitra';

if opt_est.res_anal == 1
    n_lags = 10;
    c = xcorr(model.residuals,n_lags,'coeff');
    c = c(n_lags+1:end);
    lags = 1:(n_lags+1);
    figure(5);
    scnsize = get(0,'ScreenSize');
    set(5,'OuterPosition',[scnsize(3)/2,...
            scnsize(4) * (2/3),...
            scnsize(3)/2,...
            scnsize(4)/3]);
    stem(lags,c);
end
