function network = rn_init(opt_est)
% Method initializing a multiple input single output RN
% Note: echo state property satisfied by spectral radius < alpha

% INPUT
% opt an option structure with:
%     n_in: dimension of the input
%     n_units: number of internal units (n_units > 2 * n_in)
%     alpha: spectral radius for the RN (0 < alpha < 1)
%     f: internal state activation function (function handler)
%     f_out: output activation function (function handler)
%     f_out_inv: inverse output activation function (function handler)
%     init_W: internal weight initialization
%           'sparse' sparse random matrix 
%           'subspa' (to do)
%     spa_perc: pecentage of sparseness ((0 < spa_perc < 1)

% OUTPUT
% network with the following structure:
%     u: input
%     x: internal states
%     y: output states
%     W: internal weights (random)
%     W_in: input weights (random)
%     W_out: output weights (initially zero)
%     f: internal state activation function
%     f_out: output activation function
%     f_out_inv: inverse output activation function

% Model
% x(n+1) = f(W*x(n) + W_in*u(n+1))
% y(n+1) = f_out(W_out*(u(n+1),x(n+1)))

% Possible 
if opt_est.n_node < 2 * opt_est.n_in
    error('Reservoir is too small');
elseif opt_est.alpha > 1
    error('The echo state network may not be satisfied (choose alpha < 1)');
elseif opt_est.spa_perc > 1 || opt_est.spa_perc < 0
    error('Sparseness factor should be > 0 and < 1');
else
    eig_max = 0;
    while eig_max == 0
        %Initializing internal weights
        if strcmp(opt_est.init_W,'sparse')
            network.W = sprand(opt_est.n_node,opt_est.n_node,opt_est.spa_perc);
            network.W(network.W ~= 0) = network.W(network.W ~= 0) - 0.5;
        elseif strcmp(opt_est.init_W,'subspa')
            % To do
            %network.W = ...;
        end
        eig_max = max(abs(eig(full(network.W))));
        network.W = network.W / eig_max * opt_est.alpha;
    end
    
    %Initializing internal states and input-output weights
    network.W_in = 2 * rand(opt_est.n_node,opt_est.n_in) - 1;
    network.W_out = zeros(opt_est.n_node,1);
    network.x = zeros(opt_est.n_node,1);

    network.f = opt_est.f;
    network.f_out = opt_est.f_out;
    network.f_out_inv = opt_est.f_out_inv;
end
